@@ -38,7 +38,6 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
 #include <gdk/gdkkeysyms.h>
@@ -353,7 +352,7 @@ create_pixmap (GnomeUIPixmapType pixmap_type, gconstpointer pixmap_info,
 				((const char **)pixmap_info);
 			if (pixbuf != NULL) {
 				pixmap = gtk_image_new_from_pixbuf (pixbuf);
-				gdk_pixbuf_unref (pixbuf);
+				g_object_unref (G_OBJECT (pixbuf));
 			}
 		}
 
@@ -407,10 +406,10 @@ showing_pixmaps_changed_notify(GConfClient            *client,
                 GnomeUIPixmapType pixmap_type;
                 gconstpointer pixmap_info;
 
-                pixmap_type = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(mi),
-                                                                  gnome_app_helper_pixmap_type));
-                pixmap_info = gtk_object_get_data(GTK_OBJECT(mi),
-                                                  gnome_app_helper_pixmap_info);
+                pixmap_type = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(mi),
+								gnome_app_helper_pixmap_type));
+                pixmap_info = g_object_get_data(G_OBJECT(mi),
+						gnome_app_helper_pixmap_info);
 
                 pixmap = create_pixmap (pixmap_type, pixmap_info,
 					GTK_ICON_SIZE_MENU);
@@ -436,7 +435,7 @@ remove_notify_cb(GtkObject *obj, gpointer data)
         
         notify_id = GPOINTER_TO_INT(data);
 
-        conf = gtk_object_get_data(obj, gnome_app_helper_gconf_client);
+        conf = g_object_get_data (G_OBJECT (obj), gnome_app_helper_gconf_client);
 
         gconf_client_notify_remove(conf, notify_id);
 }
@@ -450,11 +449,11 @@ setup_image_menu_item (GtkWidget *mi, GnomeUIPixmapType pixmap_type,
         
         g_return_if_fail(GTK_IS_IMAGE_MENU_ITEM(mi));
 
-        gtk_object_set_data(GTK_OBJECT(mi), gnome_app_helper_pixmap_type,
-                            GINT_TO_POINTER(pixmap_type));
+        g_object_set_data(G_OBJECT(mi), gnome_app_helper_pixmap_type,
+			  GINT_TO_POINTER(pixmap_type));
 
-        gtk_object_set_data(GTK_OBJECT(mi), gnome_app_helper_pixmap_info,
-                            (gpointer)pixmap_info);
+        g_object_set_data(G_OBJECT(mi), gnome_app_helper_pixmap_info,
+			  (gpointer)pixmap_info);
 
 
 	/* make sure that things are all ready for us */
@@ -463,11 +462,11 @@ setup_image_menu_item (GtkWidget *mi, GnomeUIPixmapType pixmap_type,
         conf = gconf_client_get_default();
 
         g_object_ref (G_OBJECT (conf));
-        gtk_object_set_data_full(GTK_OBJECT(mi), gnome_app_helper_gconf_client,
-                                 conf, (GtkDestroyNotify)g_object_unref);
+        g_object_set_data_full(G_OBJECT(mi), gnome_app_helper_gconf_client,
+			       conf, g_object_unref);
 
         if (gconf_client_get_bool(conf,
-                                  "/desktop/gnome/menus/show-icons",
+                                  "/desktop/gnome/interface/menus_have_icons",
                                   NULL)) {
                 GtkWidget *pixmap;
 
@@ -482,29 +481,24 @@ setup_image_menu_item (GtkWidget *mi, GnomeUIPixmapType pixmap_type,
         }
 
         notify_id = gconf_client_notify_add(conf,
-                                            "/desktop/gnome/menus/show-icons",
+                                            "/desktop/gnome/interface/menus_have_icons",
                                             showing_pixmaps_changed_notify,
                                             mi, NULL, NULL);
 
-        gtk_signal_connect(GTK_OBJECT(mi), "destroy",
-                           GTK_SIGNAL_FUNC(remove_notify_cb),
-			   GINT_TO_POINTER(notify_id));
+        g_signal_connect(mi, "destroy",
+			 G_CALLBACK (remove_notify_cb),
+			 GINT_TO_POINTER(notify_id));
 }
 
-/* Creates  a menu item label. It will also return the underlined 
- * letter's keyval if keyval is not NULL. */
+/* Creates  a menu item label. */
 static GtkWidget *
-create_label (const char *label_text, guint *keyval)
+create_label (const char *label_text)
 {
-	guint kv;
 	GtkWidget *label;
 
 	label = gtk_accel_label_new (label_text);
-
-	kv = gtk_label_parse_uline (GTK_LABEL (label), label_text);
-	if (keyval)
-		*keyval = kv;
-
+	gtk_label_set_use_underline (GTK_LABEL (label), TRUE);
+	
 	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
 	gtk_widget_show (label);
 
@@ -522,59 +516,6 @@ setup_accelerator (GtkAccelGroup *accel_group, GnomeUIInfo *uiinfo,
 					    uiinfo->ac_mods, accel_flags);
 }
 
-#if 0
-GtkAccelGroup*
-gtk_menu_ensure_uline_accel_group (GtkMenu *menu)
-{
-  GtkAccelGroup *accel_group;
-
-  g_return_val_if_fail (GTK_IS_MENU (menu), NULL);
-
-  accel_group = gtk_object_get_data (GTK_OBJECT (menu),
-				     GtkMenu_uline_accel_group);
-  if (!accel_group)
-    {
-      accel_group = gtk_accel_group_new ();
-      gtk_accel_group_attach (accel_group, GTK_OBJECT (menu));
-      gtk_object_set_data_full (GTK_OBJECT (menu),
-				GtkMenu_uline_accel_group,
-				accel_group,
-				(GtkDestroyNotify) gtk_accel_group_unref);
-    }
-
-  return accel_group;
-}
-#endif
-
-/* Creates the accelerators for the underlined letter in a menu item's label. 
- * The keyval is what gtk_label_parse_uline() returned.  If accel_group is not 
- * NULL, then the keyval will be put with MOD1 as modifier in it (i.e. for 
- * Alt-F in the _File menu).
- */
-static void
-setup_uline_accel (GtkMenuShell  *menu_shell,
-		   GtkAccelGroup *accel_group,
-		   GtkWidget     *menu_item,
-		   guint          keyval)
-{
-#if 0 /* FIXME FIXME FIXME */
-	if (keyval != GDK_VoidSymbol) {
-		if (GTK_IS_MENU (menu_shell))
-			gtk_widget_add_accelerator (menu_item,
-						    "activate_item",
-						    gtk_menu_ensure_uline_accel_group (GTK_MENU (menu_shell)),
-						    keyval, 0,
-						    0);
-		if (GTK_IS_MENU_BAR (menu_shell) && accel_group)
-			gtk_widget_add_accelerator (menu_item,
-						    "activate_item", 
-						    accel_group,
-						    keyval, GDK_MOD1_MASK,
-						    0);
-	}
-#endif
-}
-
 /* Callback to display hint in the statusbar when a menu item is 
  * activated. For GtkStatusbar.
  */
@@ -582,8 +523,7 @@ setup_uline_accel (GtkMenuShell  *menu_shell,
 static void
 put_hint_in_statusbar(GtkWidget* menuitem, gpointer data)
 {
-	gchar* hint = gtk_object_get_data(GTK_OBJECT(menuitem),
-					  apphelper_statusbar_hint);
+	gchar* hint = g_object_get_data(G_OBJECT(menuitem), apphelper_statusbar_hint);
 	GtkWidget* bar = data;
 	guint id;
 	
@@ -629,19 +569,19 @@ install_menuitem_hint_to_statusbar(GnomeUIInfo* uiinfo, GtkStatusbar* bar)
 
   if (uiinfo->hint)
     {
-      gtk_object_set_data (GTK_OBJECT(uiinfo->widget),
-                           apphelper_statusbar_hint,
-                           (gpointer)L_(uiinfo->hint));
+      g_object_set_data (G_OBJECT (uiinfo->widget),
+			 apphelper_statusbar_hint,
+			 (gpointer)(L_(uiinfo->hint)));
 
-      gtk_signal_connect (GTK_OBJECT (uiinfo->widget),
-                          "select",
-                          GTK_SIGNAL_FUNC(put_hint_in_statusbar),
-                          bar);
+      g_signal_connect (G_OBJECT (uiinfo->widget),
+			"select",
+			G_CALLBACK(put_hint_in_statusbar),
+			bar);
       
-      gtk_signal_connect (GTK_OBJECT (uiinfo->widget),
-                          "deselect",
-                          GTK_SIGNAL_FUNC(remove_hint_from_statusbar),
-                          bar);
+      g_signal_connect (G_OBJECT (uiinfo->widget),
+			"deselect",
+			G_CALLBACK(remove_hint_from_statusbar),
+			bar);
     }
 }        
 
@@ -699,8 +639,7 @@ gnome_app_install_statusbar_menu_hints (GtkStatusbar* bar,
 static void
 put_hint_in_appbar(GtkWidget* menuitem, gpointer data)
 {
-  gchar* hint = gtk_object_get_data (GTK_OBJECT(menuitem),
-                                     "apphelper_appbar_hint");
+  gchar* hint = g_object_get_data (G_OBJECT(menuitem), "apphelper_appbar_hint");
   GtkWidget* bar = data;
 
   g_return_if_fail (hint != NULL);
@@ -950,11 +889,9 @@ create_menu_item (GtkMenuShell       *menu_shell,
 		  GSList            **radio_group,
 		  GnomeUIBuilderData *uibdata,
 		  GtkAccelGroup      *accel_group,
-		  gboolean	      uline_accels,
 		  gint		      pos)
 {
 	GtkWidget *label;
-	guint keyval;
 	int type;
 	
 	/* Translate configurable menu items to normal menu items. */
@@ -973,7 +910,7 @@ create_menu_item (GtkMenuShell       *menu_shell,
 	case GNOME_APP_UI_SUBTREE_STOCK:
 		if (is_radio) {
 			uiinfo->widget = gtk_radio_menu_item_new (*radio_group);
-			*radio_group = gtk_radio_menu_item_group
+			*radio_group = gtk_radio_menu_item_get_group
 				(GTK_RADIO_MENU_ITEM (uiinfo->widget));
 		} else {
 
@@ -1019,24 +956,15 @@ create_menu_item (GtkMenuShell       *menu_shell,
 	/* Don't use gettext on the empty string since gettext will map
 	 * the empty string to the header at the beginning of the .pot file. */
 
-	label = create_label ( uiinfo->label [0] == '\0'?
+	label = create_label ( uiinfo->label == NULL?
 			       "":(uiinfo->type == GNOME_APP_UI_SUBTREE_STOCK ?
-				   D_(uiinfo->label):L_(uiinfo->label)),
-			       &keyval);
+				   D_(uiinfo->label):L_(uiinfo->label)));
 
 	gtk_container_add (GTK_CONTAINER (uiinfo->widget), label);
 
 	gtk_accel_label_set_accel_widget (GTK_ACCEL_LABEL (label), 
 					  uiinfo->widget);
 	
-	/* setup underline accelerators
-	 */
-	if (uline_accels)
-		setup_uline_accel (menu_shell,
-				   accel_group,
-				   uiinfo->widget,
-				   keyval);
-
 	/* install global accelerator
 	 */
 	{
@@ -1071,13 +999,17 @@ create_menu_item (GtkMenuShell       *menu_shell,
 
 		/* g_print ("######## menu item path: %s\n", gstring->str); */
 		
-		/* the item factory cares about installing the correct accelerator */
+		/* associate the key combo with the accel path */
+		if (uiinfo->accelerator_key)
+			gtk_accel_map_add_entry (gstring->str,
+						 uiinfo->accelerator_key,
+						 uiinfo->ac_mods);
+
+		/* associate the accel path with the menu item */
+		if (GTK_IS_MENU (menu_shell))
+			gtk_menu_item_set_accel_path(GTK_MENU_ITEM (uiinfo->widget),
+						     gstring->str);
 		
-		gtk_item_factory_add_foreign (uiinfo->widget,
-					      gstring->str,
-					      accel_group,
-					      uiinfo->accelerator_key,
-					      uiinfo->ac_mods);
 		g_string_free (gstring, TRUE);
 
 		if (!save_accels_id)
@@ -1086,13 +1018,10 @@ create_menu_item (GtkMenuShell       *menu_shell,
 	
 	/* Set toggle information, if appropriate */
 	
-	if ((uiinfo->type == GNOME_APP_UI_TOGGLEITEM) || is_radio) {
-		gtk_check_menu_item_set_show_toggle
-			(GTK_CHECK_MENU_ITEM(uiinfo->widget), TRUE);
+	if ((uiinfo->type == GNOME_APP_UI_TOGGLEITEM) || is_radio)
 		gtk_check_menu_item_set_active
 			(GTK_CHECK_MENU_ITEM (uiinfo->widget), FALSE);
-	}
-	
+
 	/* Connect to the signal and set user data */
 	
 	type = uiinfo->type;
@@ -1131,7 +1060,7 @@ create_radio_menu_items (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo,
 		case GNOME_APP_UI_ITEM:
 			create_menu_item (menu_shell, uiinfo, TRUE,
 					  &group, uibdata, 
-					  accel_group, FALSE, pos);
+					  accel_group, pos);
 			pos++;
 			break;
 
@@ -1273,8 +1202,7 @@ static void
 help_view_display_callback (GtkWidget *w, gpointer data)
 {
 	char *section = data;
-	char *docname = gtk_object_get_data
-		(GTK_OBJECT (w), "docname");
+	char *docname = g_object_get_data (G_OBJECT (w), "docname");
 
 	/* FIXME: handle errors somehow */
 	if (docname == NULL ||
@@ -1308,26 +1236,34 @@ create_help_entries (GtkMenuShell *menu_shell, GnomeUIInfo *uiinfo, gint pos)
 	for (cur = topics; cur && cur->next; cur = cur->next->next) {
 		GtkWidget *item;
 		GtkWidget *label;
-		guint keyval;
 
 		item = gtk_menu_item_new ();
-		label = create_label (cur->data, &keyval);
+		label = create_label (cur->data);
 		g_free(cur->data);
 		cur->data = NULL;
 
 		gtk_container_add (GTK_CONTAINER (item), label);
-		setup_uline_accel (menu_shell, NULL, item, keyval);
 /*		gtk_widget_lock_accelerators (item); */
 
-		gtk_object_set_data_full (GTK_OBJECT (item), "docname",
-					  g_strdup (uiinfo->moreinfo),
-					  g_free);
+		g_object_set_data_full (G_OBJECT (item), "docname",
+					g_strdup (uiinfo->moreinfo),
+					g_free);
 
+		g_signal_connect_closure_by_id (item,
+						g_signal_lookup ("activate", G_OBJECT_TYPE (item)),
+						0,
+						g_cclosure_new (
+							G_CALLBACK (help_view_display_callback),
+							cur->next->data,
+							(GClosureNotify) g_free),
+						FALSE);
+#if 0
 		gtk_signal_connect_full
 			(GTK_OBJECT (item), "activate",
 			 GTK_SIGNAL_FUNC (help_view_display_callback), NULL,
 			 cur->next->data, g_free,
 			 FALSE, FALSE);
+#endif
 		cur->next->data = NULL;
 
 		gtk_menu_shell_insert (menu_shell, item, pos);
@@ -1375,13 +1311,14 @@ do_ui_signal_connect (GnomeUIInfo        *uiinfo,
 		saved_data->destroy_func = uibdata->destroy_func;
 		saved_data->user_data = uibdata->data ? uibdata->data : uiinfo->user_data;
 
-		gtk_signal_connect_full (GTK_OBJECT (uiinfo->widget), 
-					 signal_name,
-					 GTK_SIGNAL_FUNC (ui_relay_callback),
-					 NULL,
-					 saved_data,
-					 (GtkDestroyNotify) ui_destroy_callback,
-					 FALSE, FALSE);
+		g_signal_connect_closure_by_id (uiinfo->widget, 
+						g_signal_lookup (signal_name, G_OBJECT_TYPE (uiinfo->widget)),
+						0,
+						g_cclosure_new (
+							G_CALLBACK (ui_relay_callback),
+							saved_data,
+							(GClosureNotify) ui_destroy_callback),
+						FALSE);
 	
 	} else if (uiinfo->moreinfo)
 		gtk_signal_connect (GTK_OBJECT (uiinfo->widget), 
@@ -1515,6 +1452,13 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 
 	orig_uibdata = uibdata;
 
+	/* if it is a GtkMenu, make sure the accel group is associated
+	 * with the menu */
+	if (GTK_IS_MENU (menu_shell) &&
+	    GTK_MENU (menu_shell)->accel_group == NULL)
+			gtk_menu_set_accel_group (GTK_MENU (menu_shell),
+						  accel_group);
+
 	for (; uiinfo->type != GNOME_APP_UI_ENDOFINFO; uiinfo++)
 		switch (uiinfo->type) {
 		case GNOME_APP_UI_BUILDER_DATA:
@@ -1544,13 +1488,11 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 			if (uiinfo->type == GNOME_APP_UI_SUBTREE_STOCK)
 				create_menu_item (menu_shell, uiinfo, FALSE,
 						  NULL, uibdata, 
-						  accel_group, uline_accels,
-						  pos);
+						  accel_group, pos);
 			else
 				create_menu_item (menu_shell, uiinfo, FALSE,
 						  NULL, uibdata, 
-						  accel_group, uline_accels,
-						  pos);
+						  accel_group, pos);
 			
 			if (uiinfo->type == GNOME_APP_UI_SUBTREE ||
 			    uiinfo->type == GNOME_APP_UI_SUBTREE_STOCK) {
@@ -1569,7 +1511,7 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 					 accel_group, uline_accels, 0);
 
 				/* FIXME: make this runtime configurable */ 
-				if (gnome_gconf_get_bool ("/desktop/gnome/interface/menus-have-tearoff")) {
+				if (gnome_gconf_get_bool ("/desktop/gnome/interface/menus_have_tearoff")) {
 					tearoff = gtk_tearoff_menu_item_new ();
 					gtk_widget_show (tearoff);
 					gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), tearoff);
@@ -1593,14 +1535,6 @@ gnome_app_fill_menu_custom (GtkMenuShell       *menu_shell,
 	/* Make the end item contain a pointer to the parent menu shell */
 
 	uiinfo->widget = GTK_WIDGET (menu_shell);
-
-#ifdef FIXME
-	/* Configure menu to gnome preferences, if possible.
-	 * (sync to gnome-app.c:gnome_app_set_menus) */
-	if ( ! gnome_gconf_get_bool ("/desktop/gnome/interface/menubar-relief") &&
-	     GTK_IS_MENU_BAR (menu_shell))
-		gtk_menu_bar_set_shadow_type (GTK_MENU_BAR (menu_shell), GTK_SHADOW_NONE);
-#endif
 }
 
 
@@ -1764,7 +1698,7 @@ gnome_app_create_menus_custom (GnomeApp *app, GnomeUIInfo *uiinfo,
 				    app->accel_group, TRUE, 0);
 
 	/* FIXME: make this runtime configurable */ 
-	if (gnome_gconf_get_bool ("/desktop/gnome/interface/menus-have-tearoff")) {
+	if (gnome_gconf_get_bool ("/desktop/gnome/interface/menus_have_tearoff")) {
 		gchar *app_name;
 
 		app_name = GTK_WINDOW (app)->title;
@@ -1838,7 +1772,7 @@ create_toolbar_item (GtkToolbar *toolbar, GnomeUIInfo *uiinfo, int is_radio,
 
 	/* Set the accelerator and connect to the signal */
 
-	if (is_radio || (uiinfo->type == GNOME_APP_UI_TOGGLEITEM)) {
+	if (is_radio) {
 		setup_accelerator (accel_group, uiinfo, "toggled", 0);
 		(* uibdata->connect_func) (uiinfo, "toggled", uibdata);
 	} else {
@@ -2188,7 +2122,6 @@ gnome_app_find_menu_pos (GtkWidget *parent, const gchar *path, gint *pos)
 	const gchar *transl;
 	gint p;
 	int  path_len;
-	int  stripped_path_len;
 	
 	g_return_val_if_fail (parent != NULL, NULL);
 	g_return_val_if_fail (path != NULL, NULL);
@@ -2221,11 +2154,6 @@ gnome_app_find_menu_pos (GtkWidget *parent, const gchar *path, gint *pos)
 	transl = L_(part);
 	path_len = strlen(transl);
 
-	stripped_path_len = path_len;
-	for ( p = 0; p < path_len; p++ )
-	        if( transl[p] == '_' )
-		        stripped_path_len--;
-		
 	p = 0;
 
 	while (children){
@@ -2242,8 +2170,7 @@ gnome_app_find_menu_pos (GtkWidget *parent, const gchar *path, gint *pos)
 			label = GTK_LABEL (item->child)->label;
 		else
 			label = NULL; /* something that we just can't handle */
-		if (label && (stripped_path_len == strlen (label)) &&
-		    (g_strncmp_ignore_char (transl, label, path_len, '_') == 0)){
+		if (label && (g_strncmp_ignore_char (transl, label, path_len, '_') == 0)){
 			if (name_end == NULL) {
 				*pos = p;
 				g_free(part);
@@ -2506,102 +2433,6 @@ gnome_app_helper_gettext (const gchar *str)
 }
 #endif
 
-static void
-set_bevels(BonoboDockItem *dock_item, gboolean bevels)
-{
-        if (bevels) {
-                gtk_container_set_border_width (GTK_CONTAINER (dock_item), 1);
-                bonobo_dock_item_set_shadow_type (BONOBO_DOCK_ITEM (dock_item),
-                                                 GTK_SHADOW_OUT);
-        } else {
-                gtk_container_set_border_width (GTK_CONTAINER (dock_item), 0);
-                bonobo_dock_item_set_shadow_type (BONOBO_DOCK_ITEM (dock_item),
-                                                 GTK_SHADOW_NONE);
-        }
-}
-
-static void
-dockitem_bevels_changed_notify(GConfClient            *client,
-                               guint                   cnxn_id,
-			       GConfEntry             *entry,
-                               gpointer                user_data)
-{
-        gboolean bevels = TRUE;
-        GtkWidget *w = user_data;
-        BonoboDockItem *dock_item = BONOBO_DOCK_ITEM(w);
-	GConfValue *value = gconf_entry_get_value (entry);
-
-        if (value &&
-            value->type == GCONF_VALUE_BOOL) {
-                bevels = gconf_value_get_bool(value);
-        }
-
-        set_bevels(dock_item, bevels);
-}
-
-static void
-set_separators(GtkToolbar *toolbar, gboolean separators)
-{
-#ifdef FIXME
-        if (separators) {
-                gtk_toolbar_set_space_style (toolbar, GTK_TOOLBAR_SPACE_LINE);
-                gtk_toolbar_set_space_size (toolbar, GNOME_PAD_SMALL * 2);
-        } else {
-                gtk_toolbar_set_space_style (toolbar, GTK_TOOLBAR_SPACE_EMPTY);
-                gtk_toolbar_set_space_size (toolbar, GNOME_PAD_SMALL);
-        }
-#endif
-}
-
-static void
-toolbar_separators_changed_notify(GConfClient            *client,
-                                  guint                   cnxn_id,
-				  GConfEntry             *entry,
-                                  gpointer                user_data)
-{
-        gboolean separators = TRUE;
-        GtkWidget *w = user_data;
-        GtkToolbar *toolbar = GTK_TOOLBAR(w);
-	GConfValue *value = gconf_entry_get_value (entry);
-
-        if (value &&
-            value->type == GCONF_VALUE_BOOL) {
-                separators = gconf_value_get_bool(value);
-        }
-
-        set_separators(toolbar, separators);
-}
-
-static GConfEnumStringPair toolbar_reliefs[] = {
-        { GTK_RELIEF_NORMAL, "normal" },
-        { GTK_RELIEF_NONE, "none" },
-        { GTK_RELIEF_HALF, "half" }
-};
-
-static void
-toolbar_relief_changed_notify(GConfClient            *client,
-                              guint                   cnxn_id,
-			      GConfEntry             *entry,
-                              gpointer                user_data)
-{
-#ifdef FIXME
-        GtkReliefStyle style = GTK_RELIEF_NONE;
-        GtkWidget *w = user_data;
-        GtkToolbar *toolbar = GTK_TOOLBAR(w);
-	GConfValue *value = gconf_entry_get_value (entry);
-
-        if (value &&
-            value->type == GCONF_VALUE_STRING &&
-            gconf_value_get_string(value) != NULL) {
-                gconf_string_to_enum(toolbar_reliefs,
-                                     gconf_value_get_string(value),
-                                     (gint*)&style);
-        }
-
-        gtk_toolbar_set_button_relief(toolbar, style);
-#endif
-}
-
 static GConfEnumStringPair toolbar_styles[] = {
         { GTK_TOOLBAR_TEXT, "text" },
         { GTK_TOOLBAR_ICONS, "icons" },
@@ -2636,7 +2467,7 @@ per_app_toolbar_style_changed_notify(GConfClient            *client,
                 gchar *str;
 
                 str = gconf_client_get_string(client,
-                                              "/desktop/gnome/toolbars/style",
+                                              "/desktop/gnome/interface/toolbar_style",
                                               NULL);
 
                 if (str) {
@@ -2666,7 +2497,7 @@ toolbar_style_changed_notify(GConfClient            *client,
 	GConfValue *value;
 
         /* Check for app-specific override */
-        per_app_key = gnome_gconf_get_gnome_libs_settings_relative("toolbar-style");
+        per_app_key = gnome_gconf_get_gnome_libs_settings_relative("toolbar_style");
         str = gconf_client_get_string(client, per_app_key, NULL);
         g_free(per_app_key);
 
@@ -2694,6 +2525,166 @@ toolbar_style_changed_notify(GConfClient            *client,
         gtk_toolbar_set_style(toolbar, style);
 }
 
+static void
+style_menu_item_activated (GtkWidget *item, GtkToolbarStyle style)
+{
+	GConfClient *conf;
+	char *key;
+	int i;
+	
+	key = gnome_gconf_get_gnome_libs_settings_relative("toolbar_style");
+	conf = gconf_client_get_default();
+
+	/* Set our per-app toolbar setting */
+	for (i = 0; i < G_N_ELEMENTS (toolbar_styles); i++) {
+		if (toolbar_styles[i].enum_value == style) {
+			gconf_client_set_string (conf, key, toolbar_styles[i].str, NULL);
+			break;
+		}
+	}
+	g_free (key);
+}
+
+static void
+global_menu_item_activated (GtkWidget *item)
+{
+	GConfClient *conf;
+	char *key;
+	
+	key = gnome_gconf_get_gnome_libs_settings_relative("toolbar_style");
+	conf = gconf_client_get_default();
+	
+	/* Unset the per-app toolbar setting */
+	gconf_client_unset (conf, key, NULL);
+	g_free (key);
+
+}
+
+static void
+create_and_popup_toolbar_menu (GdkEventButton *event)
+{
+	GtkWidget *menu;
+	GtkWidget *item, *both_item, *icons_item, *text_item, *global_item;
+	GSList *group;
+	char *both, *icons, *text, *global;
+	char *str, *key;
+	GtkToolbarStyle toolbar_style;
+	GConfClient *conf;
+	
+	group = NULL;
+	toolbar_style = GTK_TOOLBAR_BOTH;
+	menu = gtk_menu_new ();
+
+	both = _("Both text and icons");
+	icons = _("Icons only");
+	text = _("Text only");
+
+	both_item = gtk_radio_menu_item_new_with_label (group, both);
+	g_signal_connect (both_item, "activate",
+			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_BOTH));
+	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (both_item));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), both_item);
+
+	icons_item = gtk_radio_menu_item_new_with_label (group, icons);
+	g_signal_connect (icons_item, "activate",
+			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_ICONS));
+	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (icons_item));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), icons_item);
+
+	text_item = gtk_radio_menu_item_new_with_label (group, text);
+	g_signal_connect (text_item, "activate",
+			  G_CALLBACK (style_menu_item_activated), GINT_TO_POINTER (GTK_TOOLBAR_TEXT));
+
+	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (text_item));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), text_item);
+
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	/* Get global setting */
+	conf = gconf_client_get_default();
+	str = gconf_client_get_string(conf,
+				      "/desktop/gnome/interface/toolbar_style",
+				      NULL);
+	
+	if (str != NULL) {
+		gconf_string_to_enum(toolbar_styles,
+				     str,
+				     (gint*)&toolbar_style);
+		g_free(str);
+	}
+
+	switch (toolbar_style) {
+	case GTK_TOOLBAR_BOTH:
+		str = both;
+		break;
+	case GTK_TOOLBAR_ICONS:
+		str = icons;
+		break;
+	case GTK_TOOLBAR_TEXT:
+		str = text;
+		break;
+	default:
+		g_assert_not_reached ();
+	}
+
+	global = g_strdup_printf (_("Use desktop default (%s)"),
+				  str);
+	global_item = gtk_radio_menu_item_new_with_label (group, global);
+	g_signal_connect (global_item, "activate",
+			  G_CALLBACK (global_menu_item_activated), NULL);
+	group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (global_item));	
+	g_free (global);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), global_item);
+
+	gtk_widget_show_all (menu);
+
+	/* Now select the correct menu according to our preferences */
+	key = gnome_gconf_get_gnome_libs_settings_relative("toolbar_style");
+	str = gconf_client_get_string(conf, key, NULL);
+
+	if (str == NULL) {
+		/* We have no per-app setting, so the global one must be right. */
+		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (global_item), TRUE);
+	}
+	else {
+		gconf_string_to_enum(toolbar_styles,
+				     str,
+				     (gint*)&toolbar_style);
+		
+		/* We have a per-app setting, find out which one it is */
+		switch (toolbar_style) {
+		case GTK_TOOLBAR_BOTH:
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (both_item), TRUE);
+			break;
+		case GTK_TOOLBAR_ICONS:
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (icons_item), TRUE);
+			break;
+		case GTK_TOOLBAR_TEXT:
+			gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (text_item), TRUE);
+			break;
+		default:
+			g_assert_not_reached ();
+		}
+
+		g_free (str);
+	}
+	
+	g_free (key);
+	gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL, event->button, event->time);
+	
+}
+
+static int
+button_press (GtkWidget *dock, GdkEventButton *event, gpointer user_data)
+{
+	if (event->button == 3) {
+		create_and_popup_toolbar_menu (event);
+	}
+
+	return FALSE;
+}
+
 /**
  * gnome_app_setup_toolbar
  * @toolbar: Pointer to #GtkToolbar widget
@@ -2711,6 +2702,7 @@ gnome_app_setup_toolbar (GtkToolbar *toolbar,
 
 	/* make sure that things are all ready for us */
 	gnomeui_gconf_lazy_init ();
+	
 
         conf = gconf_client_get_default();
 
@@ -2721,77 +2713,15 @@ gnome_app_setup_toolbar (GtkToolbar *toolbar,
         /* Attach GConf settings */
 
         if (dock_item != NULL) { /* Dock item bevel */
-                gboolean bevels = TRUE;
-                guint notify_id;
-
 		g_object_ref (G_OBJECT (conf));
 		gtk_object_set_data_full(GTK_OBJECT(dock_item), gnome_app_helper_gconf_client,
 					 conf, (GtkDestroyNotify)g_object_unref);
                 
-                bevels = gconf_client_get_bool(conf,
-                                               "/desktop/gnome/toolbars/bevels",
-                                               NULL);
-                
-                notify_id = gconf_client_notify_add(conf,
-                                                    "/desktop/gnome/toolbars/bevels",
-                                                    dockitem_bevels_changed_notify,
-                                                    dock_item, NULL, NULL);
-                
-                gtk_signal_connect(GTK_OBJECT(dock_item), "destroy",
-                                   GTK_SIGNAL_FUNC(remove_notify_cb),
-				   GINT_TO_POINTER(notify_id));
+		gtk_signal_connect (GTK_OBJECT (dock_item),
+				    "button_press_event",
+				    GTK_SIGNAL_FUNC (button_press),
+				    NULL);
 
-                set_bevels(dock_item, bevels);
-        }
-        
-        { /* Toolbar line separators */
-                gboolean separators = TRUE;
-                guint notify_id;
-                
-                separators = gconf_client_get_bool(conf,
-                                                   "/desktop/gnome/toolbars/separators",
-                                                   NULL);
-                
-                notify_id = gconf_client_notify_add(conf,
-                                                    "/desktop/gnome/toolbars/separators",
-                                                    toolbar_separators_changed_notify,
-                                                    toolbar, NULL, NULL);
-
-                gtk_signal_connect(GTK_OBJECT(toolbar), "destroy",
-                                   GTK_SIGNAL_FUNC(remove_notify_cb),
-				   GINT_TO_POINTER(notify_id));
-
-                set_separators(toolbar, separators);
-        }
-        
-        { /* Toolbar button relief */
-                GtkReliefStyle relief_style = GTK_RELIEF_NONE;
-                guint notify_id;
-                gchar *str;
-                
-                str = gconf_client_get_string(conf,
-                                              "/desktop/gnome/toolbars/relief",
-                                              NULL);
-
-                if (str != NULL) {
-                        gconf_string_to_enum(toolbar_reliefs,
-                                             str,
-                                             (gint*)&relief_style);
-                        g_free(str);
-                }
-                
-                notify_id = gconf_client_notify_add(conf,
-                                                    "/desktop/gnome/toolbars/relief",
-                                                    toolbar_relief_changed_notify,
-                                                    toolbar, NULL, NULL);
-
-                gtk_signal_connect(GTK_OBJECT(toolbar), "destroy",
-                                   GTK_SIGNAL_FUNC(remove_notify_cb),
-				   GINT_TO_POINTER(notify_id));
-
-#ifdef FIXME
-                gtk_toolbar_set_button_relief (toolbar, relief_style);
-#endif
         }
         
         { /* Toolbar Style */
@@ -2806,8 +2736,8 @@ gnome_app_setup_toolbar (GtkToolbar *toolbar,
                 gboolean got_it = FALSE;
 
                 /* Try per-app key */
-                per_app_key = gnome_gconf_get_gnome_libs_settings_relative("toolbar-style");
-
+                per_app_key = gnome_gconf_get_gnome_libs_settings_relative("toolbar_style");
+		
                 str = gconf_client_get_string(conf,
                                               per_app_key,
                                               NULL);
@@ -2824,7 +2754,7 @@ gnome_app_setup_toolbar (GtkToolbar *toolbar,
                 /* Try global default */
                 if (!got_it) {
                         str = gconf_client_get_string(conf,
-                                                      "/desktop/gnome/toolbars/style",
+                                                      "/desktop/gnome/interface/toolbar_style",
                                                       NULL);
 
                         if (str != NULL) {
@@ -2836,7 +2766,7 @@ gnome_app_setup_toolbar (GtkToolbar *toolbar,
                 }
                 
                 notify_id = gconf_client_notify_add(conf,
-                                                    "/desktop/gnome/toolbars/style",
+                                                    "/desktop/gnome/interface/toolbar_style",
                                                     toolbar_style_changed_notify,
                                                     toolbar, NULL, NULL);
 
