@@ -58,6 +58,13 @@
 #include <gtk/gtkmain.h>
 #include <gtk/gtkaccelmap.h>
 
+/* for sound events */
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtktogglebutton.h>
+#include <gtk/gtkcheckmenuitem.h>
+
 /*****************************************************************************
  * libgnomeui
  *****************************************************************************/
@@ -259,8 +266,6 @@ libgnomeui_instance_init (GnomeProgram *program, GnomeModuleInfo *mod_info)
 static void
 libgnomeui_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 {
-        gboolean ctype_set;
-        char *ctype, *old_ctype = NULL;
         gboolean do_crash_dialog = TRUE;
         const char *envar;
 
@@ -274,36 +279,122 @@ libgnomeui_pre_args_parse(GnomeProgram *app, GnomeModuleInfo *mod_info)
 
         if(do_crash_dialog)
                 libgnomeui_segv_setup (app, FALSE);
+}
 
-        /* Begin hack to propogate an en_US locale into Gtk+ if LC_CTYPE=C, so that non-ASCII
-           characters will display for as many people as possible. Related to bug #1979 */
-        ctype = setlocale (LC_CTYPE, NULL);
+static gboolean
+relay_gtk_signal(GSignalInvocationHint *hint,
+		 guint n_param_values,
+		 const GValue *param_values,
+		 gchar *signame)
+{
+#ifdef HAVE_ESD
+  char *pieces[3] = {"gtk-events", NULL, NULL};
+  static GQuark disable_sound_quark = 0;
+  GObject *object = g_value_get_object (&param_values[0]);
 
-        if (strcmp (ctype, "C") == 0) {
-                old_ctype = g_strdup (g_getenv ("LC_CTYPE"));
-                gnome_setenv ("LC_CTYPE", "en_US", TRUE);
-                ctype_set = TRUE;
-        } else {
-                ctype_set = FALSE;
-	}
+  pieces[1] = signame;
 
-        gtk_set_locale ();
+  if(!disable_sound_quark)
+    disable_sound_quark = g_quark_from_static_string("gnome_disable_sound_events");
 
-        if (ctype_set) {
-                if (old_ctype) {
-			gnome_setenv ("LC_CTYPE", old_ctype, TRUE);
-                        g_free (old_ctype);
-                } else {
-			gnome_unsetenv ("LC_CTYPE");
+  if(g_object_get_qdata (object, disable_sound_quark))
+    return TRUE;
+
+  if(GTK_IS_WIDGET(object)) {
+    if(!GTK_WIDGET_DRAWABLE(object))
+      return TRUE;
+
+    if(GTK_IS_MENU_ITEM(object) && GTK_MENU_ITEM(object)->submenu)
+      return TRUE;
+  }
+
+  gnome_triggers_vdo("", NULL, (const char **)pieces);
+
+  return TRUE;
+#else
+  return FALSE; /* this shouldn't even happen... */
+#endif
+}
+
+static void
+initialize_gtk_signal_relay (void)
+{
+#ifdef HAVE_ESD
+	gpointer iter_signames;
+	char *signame;
+	char *ctmp, *ctmp2;
+	
+	if (gnome_sound_connection_get () < 0)
+		return;
+	
+	if (!gnome_config_get_bool ("/sound/system/settings/event_sounds=true"))
+		return;
+	
+	ctmp = gnome_config_file ("/sound/events/gtk-events.soundlist");
+	ctmp2 = g_strconcat ("=", ctmp, "=", NULL);
+	g_free (ctmp);
+	iter_signames = gnome_config_init_iterator_sections (ctmp2);
+	gnome_config_push_prefix (ctmp2);
+	g_free (ctmp2);
+	
+	while ((iter_signames = gnome_config_iterator_next (iter_signames,
+							    &signame, NULL))) {
+		int signums [5];
+		int nsigs, i;
+		gboolean used_signame;
+
+		/*
+		 * XXX this is an incredible hack based on a compile-time
+		 * knowledge of what gtk widgets do what, rather than
+		 * anything based on the info available at runtime.
+		 */
+		if(!strcmp (signame, "activate")){
+			g_type_class_unref (g_type_class_ref (gtk_menu_item_get_type ()));
+			signums [0] = g_signal_lookup (signame, gtk_menu_item_get_type ());
+			
+			g_type_class_unref (g_type_class_ref (gtk_entry_get_type ()));
+			signums [1] = g_signal_lookup (signame, gtk_entry_get_type ());
+			nsigs = 2;
+		} else if(!strcmp(signame, "toggled")){
+			g_type_class_unref (g_type_class_ref (gtk_toggle_button_get_type ()));
+			signums [0] = g_signal_lookup (signame,
+							 gtk_toggle_button_get_type ());
+			
+			g_type_class_unref (g_type_class_ref (gtk_check_menu_item_get_type ()));
+			signums [1] = g_signal_lookup (signame,
+							 gtk_check_menu_item_get_type ());
+			nsigs = 2;
+		} else if (!strcmp (signame, "clicked")){
+			g_type_class_unref (g_type_class_ref (gtk_button_get_type ()));
+			signums [0] = g_signal_lookup (signame, gtk_button_get_type ());
+			nsigs = 1;
+		} else {
+			g_type_class_unref (g_type_class_ref (gtk_widget_get_type ()));
+			signums [0] = g_signal_lookup (signame, gtk_widget_get_type ());
+			nsigs = 1;
 		}
-        }
-        /* End hack */
+
+		used_signame = FALSE;
+		for(i = 0; i < nsigs; i++) {
+			if (signums [i] > 0) {
+				g_signal_add_emission_hook (signums [i], 0,
+							      (GSignalEmissionHook)relay_gtk_signal,
+							      signame, NULL);
+                                used_signame = TRUE;
+                        }
+                }
+
+                if(!used_signame)
+                        g_free(signame);
+	}
+	gnome_config_pop_prefix ();
+#endif
 }
 
 static void
 libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
 {
-        GnomeProgramPrivate_libgnomeui *priv = g_new0(GnomeProgramPrivate_libgnomeui, 1);
+        GnomeProgramPrivate_libgnomeui *priv;
         gchar *filename;
 
         gnome_type_init();
@@ -315,8 +406,6 @@ libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
         priv = g_object_get_qdata(G_OBJECT(program), quark_gnome_program_private_libgnomeui);
         priv->constructed = TRUE;
 
-        _gnome_stock_icons_init ();
-
         /* load the accelerators */
         filename = g_build_filename (gnome_user_accels_dir_get (),
                                      gnome_program_get_app_id (program),
@@ -325,7 +414,13 @@ libgnomeui_post_args_parse(GnomeProgram *program, GnomeModuleInfo *mod_info)
         g_free (filename);
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMEUILOCALEDIR);
-	//bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif
+
+        _gnome_stock_icons_init ();
+
+	initialize_gtk_signal_relay ();
 }
 
 static void
@@ -398,7 +493,7 @@ libgnomeui_rc_parse (GnomeProgram *program, const gchar *command)
 	/* <gnomedatadir>/gtkrc */
         file = gnome_program_locate_file (gnome_program_get (),
                                           GNOME_FILE_DOMAIN_DATADIR,
-                                          "gtkrc", TRUE, NULL);
+                                          "gtkrc-2.0", TRUE, NULL);
   	if (file) {
   		gtk_rc_add_default_file (file); 
 		g_free (file);
@@ -414,7 +509,7 @@ libgnomeui_rc_parse (GnomeProgram *program, const gchar *command)
         }
 	
 	/* ~/.gnome/gtkrc */
-	file = gnome_util_home_file("gtkrc");
+	file = gnome_util_home_file("gtkrc-2.0");
 	if (file) {
 		gtk_rc_add_default_file (file);
 		g_free (file);
